
pro ssw_batch, command_list, nodelist=nodelist, user=user, remove=remove, $
	batchdir=batchdir, jobname=jobname, logdir=logdir, logfile=logfile, $
	mail=mail, nodelete=nodelete, proname=proname, logname=logname, $
	caller=caller
;+
;   Name: ssw_batch
;
;   Purpose: make an idl 'batch' job and 'submit' it (VMS terminology -
;            something like a detached process under UNIX/Linux etc.)
;            (puts and end statement on your command list and calls ssw_batch)
;
;   Input Parameters:
;      command_list - one or more idl statements to execute OR filename
;
;   Keyword Parameters:
;      nodelist - one or more nodes to submit job on (will choose 'best' node)
;      user     - user for remote job (ignored if no nodelist)
;      remove   - if set, remove selected node from nodelist (for distribuinting jobs)
;      batchdir - directory for job (added to IDL !path) - default = $DIR_SITE_BATCH
;      logdir   - if set, directory for log file
;      nodelete - if set, do not delete job & log after succesful completion
;      caller    - string name of calling routine (passed to pr_status)
;
;   Calling Sequence:
;      ssw_batch, command_list [nodelist=nodelist, user=user, batchdir=batchdir, $
;			      [logdir=logdir, logfile=logfile]
;
;   Calling Examples:
;      ssw_batch, 'daily_forecast' , nodelist=['flare13','flare14','flare15']
;      (runs the idl command 'daily_forecast' as a background task on the
;      (least utiilized machine in the nodelist)
;      
;   Side Effects:
;      an idl child process is submitted in the background on the specified
;      or best node available.  
;
;   History:
;      29-Sep-1994 (SLF) written to facilitated distributed processing
;       1-Oct-1994 (SLF) added job file capability 
;       2-Oct-1994 (SLF) added CALLER keyword (passed to pr_status)
;      20-Apr-2000 S.L.Freeland - go_batch.pro->ssw_batch.pro
;
;   Restrictions:
;     UNIX only for now..
;-
;
common ssw_batch_blk, count		; used for uniq jobnames

if not data_chk(command_list,/string) then begin
   message,/info,"Must supply command, command list, or filename...
   return
endif

comm_list=command_list

if n_elements(count) eq 0 then count=0 else count=(count+1) mod 100

filein= n_elements(comm_list) eq 1 and file_exist(comm_list)
if filein then begin
   message,/info,"job file name supplied"
   nodelete=1
   break_file,comm_list(0),logdir,batchdir,jobname,ext,ver
   comm_list=rd_tfile(comm_list(0))      
   filein=1
endif   


; the value of environmental $SSW_SITE_BATCH may be set in the 

if not keyword_set(batchdir) then batchdir = '$DIR_SITE_BATCH'
if not keyword_set(logdir)   then logdir   = '$SSW_SITE_LOGS'

if not file_exist(batchdir) then batchdir=curdir()

if not filein and not write_access(batchdir) then begin
   box_message,['Do not have required write access to: ' + batchdir + ' returning...']
   return
endif

local=get_host(/short)

if not keyword_set(user) then user=get_user()

if not keyword_set(jobname) then begin
   jobname=str2arr( (str2arr(comm_list(0)))(0),'_')
   jobname='gob_' + arr2str(strmid(jobname,0,1),'') + '_' + $
     strmid(get_user(),0,3) + '_' + ex2fid(ut_time(/ex)) + '_' + $
	string(count,format='(i2.2)')
endif
		
proname=concat_dir(batchdir,jobname+'.pro')
logname=concat_dir(logdir,  jobname+'.log')

remchoice=[' /remove ',' ']

cmd="nohup " + concat_dir('SSW_BIN','ssw_batch') + " "  + jobname + $
	remchoice(keyword_set(nodelete)) + logname + " &"

; make the job file if the user supplied commands instead of a filename
if not filein then begin
;  make sure there is only one end statement (dont add one if users supplies it)
   noendss=where(strupcase(strtrim(comm_list,2)) ne 'END',noendcnt)
   commands=comm_list(noendss)
;  get some status info to insert
   pr_status,header,/idldoc, caller=caller
   file_append,/new,proname,[header,commands,'end']  ; make jobfile
endif   

print,"COMMAND: " + cmd
spawn, cmd

return
end
